<?php
/**
 * W3 XooguDynamicShortcodesAdmin module
 */
if (!defined('W3TC')) {
    die();
}
class W3_XooguDynamicShortcodesAdmin
{
    //@var array $aShortcodes Holds the shortcodes to make dynamic
    protected $aShortcodes;
    
    public function run()
    {
        //W3TC doesn't save options correctly so we need to do slight fix-up
        add_action('w3tc_save_options-w3tc_extensions', array($this, 'processPOST'));
        //Get the list of shortcodes we are currently making dynamic
        $this->aShortcodes = w3tc_get_extension_config('xoogu-dynamic-shortcodes-w3tc', 'shortcodes', null, array());
        //Load the internationalization
        load_plugin_textdomain('xoogu-dynamic-shortcodes-w3tc', false, dirname(__FILE__).'/XooguDynamicShortcodes/lang/');
        //Init for displaying the admin settings page
        add_action('w3tc_extensions_admin_init', array($this, 'admin_init'));
        //Register this extension
        add_filter('w3tc_extensions', array($this, 'extension'), 10, 2);
        //Add activation and deactivation handlers to add / remove the W3TC_DYNAMIC_SECURITY constant to wp-config.php
        add_action('w3tc_activate_extension-xoogu-dynamic-shortcodes-w3tc', array($this, 'activate'));
        add_action('w3tc_deactivate_extension-xoogu-dynamic-shortcodes-w3tc', array($this, 'deactivate'));
        //Get W3TC config
        $config_global = w3_instance('W3_Config');
        //If extension has been activated then page cache or late init disabled, deactivate extension
        if (!$config_global->get_boolean('pgcache.late_init') || !$config_global->get_boolean('pgcache.enabled')) {
            w3tc_deactivate_extension('xoogu-dynamic-shortcodes-w3tc', $config_global);
        }
    }

    
    /**
     *Display settings on extension settings page
     */
    public function admin_init()
    {
        w3tc_add_settings_section(
            'xds_settings', // Unique identifier for the settings section
            __('Settings','xoogu-dynamic-shortcodes-w3tc'), // Section title
            '__return_false', // Section callback (we don't want anything)
            'xoogu-dynamic-shortcodes-w3tc' // extension id, used to uniquely identify the extension;
        );
        w3tc_register_setting('xoogu-dynamic-shortcodes-w3tc', 'shortcodes', array($this, 'processPOST'));
        w3tc_add_settings_field("shortcodes", __('Shortcodes to make dynamic','xoogu-dynamic-shortcodes-w3tc'),
            array($this, 'printSettingsShortcodes'), 'xoogu-dynamic-shortcodes-w3tc', 'xds_settings',
            array('label_for'=>"extensions_settings_xoogu-dynamic-shortcodes-w3tc_shortcodes"));
        
    }
    
    
    /**
     *Removes shortcode setting from settings when the form is saved with no shortcodes selected
     *(By default W3TC keeps the settings as they are when this happens, so this is a workaround to fix this problem)
     *@param W3_Config $oConfig The W3TC config settings object
     *@return W3_Config The updated W3TC config settings object
     */
    public function processPOST($oConfig)
    {
        //check we're on the settings page
        if (!empty($_POST['redirect']) && strstr($_POST['redirect'], 'extension=xoogu-dynamic-shortcodes-w3tc')) {
            
            //get the existing settings for all extensions
            $aExtensionsData = $oConfig->get_array('extensions.settings', array());
            //bool to hold whether any extra shortcodes were entered (and added)
            $bShortcodeAdded = false;

            //if no 'standard' shortcodes selected, we need to set our shortcodes to an empty array, since W3TC keeps the settings otherwise
            if (empty($_POST['extensions_settings_xoogu-dynamic-shortcodes-w3tc_shortcodes'])) {
                $this->aShortcodes = array();
            } else {
                $this->aShortcodes = $aExtensionsData['xoogu-dynamic-shortcodes-w3tc']['shortcodes'];
            }
            
            //parse any extra shortcodes
            if (!empty($_POST['extraShortcodes'])) {
                $aExtraShortcodes = explode(',', $_POST['extraShortcodes']);
                foreach ($aExtraShortcodes as $sExtraShortcode) {
                    $sExtraShortcode = trim($sExtraShortcode);
                    if ($sExtraShortcode && !in_array($sExtraShortcode, $this->aShortcodes)) {
                        $this->aShortcodes[] = $sExtraShortcode;
                        $bShortcodeAdded = true;
                    }
                }
                //If any shortcodes were manually entered, then update the extension settings with these
                if ($bShortcodeAdded) {
                    $aExtensionsData['xoogu-dynamic-shortcodes-w3tc']['shortcodes'] = $this->aShortcodes;
                }
            }
            
            //check if no shortcodes were selected & check the setting isn't already empty
            if (!$bShortcodeAdded && empty($_POST['extensions_settings_xoogu-dynamic-shortcodes-w3tc_shortcodes']) && !empty($aExtensionsData['xoogu-dynamic-shortcodes-w3tc']['shortcodes'])) {
                //remove the setting
                unset($aExtensionsData['xoogu-dynamic-shortcodes-w3tc']['shortcodes']);
            }
            
            $oConfig->set('extensions.settings', $aExtensionsData);
        }
        return $oConfig;
    }
    
    
    /**
     *Prints the list of shortcodes with checkboxes for making dynamic
     *@param string $sSetting The setting id
     *@param array $aArgs Any additional arguments supplied
     */
    public function printSettingsShortcodes($sSetting, $aArgs)
    {
        list($sNameBase, $sIdBase) = w3tc_get_name_and_id('xoogu-dynamic-shortcodes-w3tc', $sSetting);
        $aShortcodes = array_flip(array_merge(array_flip(array_keys($GLOBALS['shortcode_tags'])), array_flip($this->aShortcodes)));
        sort($aShortcodes);
        echo '<ul>';
        foreach ($aShortcodes as $sShortCode) {
            $sId = $sIdBase.'_'.$sShortCode;
            $sName = $sNameBase.'[]';
            echo '<li><input type="checkbox" id="'.$sId.'" name="'.$sName.'" value="'.$sShortCode.'"';
            if (!empty($this->aShortcodes) && in_array($sShortCode, $this->aShortcodes)) {
                echo ' checked="checked"';
            }
            echo ' /><label for="'.$sId.'">'.$sShortCode.'</label></li>';
        }
        echo '</ul>';
        echo '<label for="extraShortcodes" >'.__('Any shortcodes not listed above you want to add (comma separated)','xoogu-dynamic-shortcodes-w3tc').'</label>';
        echo '<input type="text" value="" id="extraShortcodes" name="extraShortcodes" />';
    }
    
    
    /**
     * Display if caching or not.
     */
    function extension_header()
    {
        echo '<p>';
        printf(__('The Dynamic Shortcodes extension is currently %s', 'xoogu-dynamic-shortcodes-w3tc'),
            '<span class="w3tc-enabled">' . __('enabled', 'w3-total-cache') . '</span>');
        echo '.</p>';
    }
    
    
    /**
     *Add W3TC_DYNAMIC_SECURITY constant to wp-config upon activation
     */
    public function activate()
    {
        //Add the W3TC_DYNAMIC_SECURITY constant
        if (!defined('W3TC_DYNAMIC_SECURITY')) {
            
            //need to include file to make w3_wp_write_to_file() available
            w3_require_once(W3TC_INC_DIR . '/functions/activation.php');

            //Based on W3_PgCacheAdminEnvironment::wp_config_add_directive()
            $config_path = w3_get_wp_config_path();
    
            $config_data = @file_get_contents($config_path);
            if ($config_data === false) {
                exit(sprintf(__('Couldn\'t locate wp_config.php. Please add the following line to your WordPress configuation file: %s', 'xoogu-dynamic-shortcodes-w3tc'), $this->wp_config_addon()));
            }
            
            $new_config_data = preg_replace(
                '~<\?(php)?~', 
                "\\0\r\n" . $this->wp_config_addon(), 
                $config_data, 
                1);
            
            $sErrMsg = sprintf(__('Edit file %s and add line:', 'xoogu-dynamic-shortcodes-w3tc'), '<strong>' . $config_path . '</strong>');
            
            if (empty($new_config_data)) {
                exit('<p>'.$sErrMsg.'</p><pre>'.$this->wp_config_addon().'</pre>');
            }
            
            if ($new_config_data != $config_data) {
                try {
                    if (!function_exists('request_filesystem_credentials')) {
                        require_once(ABSPATH.'wp-admin/includes/file.php');
                    }
                    w3_wp_write_to_file($config_path, $new_config_data);
                } catch (FilesystemOperationException $ex) {
                    //Throwing an exception as  W3_PgCacheAdminEnvironment::wp_config_add_directive does, doesn't seem to
                    //actually do anything. Instead we need to print our error message and exit, to ensure the user sees the
                    //message instead of being redirected.
                    echo $ex->getMessage();
                    echo '<p>'.$sErrMsg.'</p><pre>'.$this->wp_config_addon().'</pre>';
                    //extension automatically gets activated, so need to force deactivation
                    w3tc_deactivate_extension('xoogu-dynamic-shortcodes-w3tc',w3_instance('W3_Config'));
                    exit();
                    
                    throw new FilesystemModifyException($ex->getMessage(), $ex->credentials_form(), $sErrMsg, $config_path, $this->wp_config_addon());
                }
            }
        }
        
        //Ensure that HTML minification won't strip out the mfunc comments
        $config_global = w3_instance('W3_Config');
        $aHTMLCommentsIgnoreList = $config_global->get_array('minify.html.comments.ignore', array());
        if (!in_array('mfunc', $aHTMLCommentsIgnoreList)) {
            $aHTMLCommentsIgnoreList[] = 'mfunc';
            $config_global->set('minify.html.comments.ignore', $aHTMLCommentsIgnoreList);
            $config_global->save();
        }
        
    }

    
    /**
     *Remove W3TC_DYNAMIC_SECURITY constant from wp-config upon deactivation
     *And remove mfunc from list of HTML minify comment exceptions
     */
    public function deactivate() {
        //remove mfunc from the list of minify comment exceptions
        $config_global = w3_instance('W3_Config');
        $aHTMLCommentsIgnoreList = $config_global->get_array('minify.html.comments.ignore', array());
        $aHTMLCommentsIgnoreList = array_diff($aHTMLCommentsIgnoreList, array('mfunc'));
        $config_global->set('minify.html.comments.ignore', $aHTMLCommentsIgnoreList);
        $config_global->save();
        
        //Remove W3TC_DYNAMIC_SECURITY constant
        if (defined('W3TC_DYNAMIC_SECURITY')) {
            
            //need to include file to make w3_wp_write_to_file() available
            w3_require_once(W3TC_INC_DIR . '/functions/activation.php');
            
            //Based on W3_PgCacheAdminEnvironment::wp_config_add_directive()
            $config_path = w3_get_wp_config_path();
    
            $config_data = @file_get_contents($config_path);
            if ($config_data === false) {
                return;
            }
            
            //Based on W3_PgCacheAdminEnvironment::wp_config_remove_from_content()
            $new_config_data = preg_replace(
                //"~(\\/\\/\\s*)?define\\s*\\(\\s*['\"]?W3TC_DYNAMIC_SECURITY['\"]?\\s*,.*?\\)\\s*;+\\r?\\n?~is",
                "/^\\s*define\\(\'W3TC_DYNAMIC_SECURITY\'.*\\r?\\n?/m",
                '', $config_data);
            
            if (empty($new_config_data)) {
                return;
            }
    
            if ($new_config_data != $config_data) {
                try {
                    if (!function_exists('request_filesystem_credentials')) {
                        require_once(ABSPATH.'wp-admin/includes/file.php');
                    }
                    w3_wp_write_to_file($config_path, $new_config_data);
                } catch (FilesystemOperationException $ex) {
                    //Throwing an exception as  W3_PgCacheAdminEnvironment::wp_config_remove_from_content does, doesn't seem to
                    //actually do anything. Instead we need to print our error message and exit, to ensure the user sees the
                    //message instead of being redirected.
                    echo $ex->getMessage();
                    echo '<p>'.sprintf(__('Edit file %s and remove line:', 'xoogu-dynamic-shortcodes-w3tc'), '<strong>' . $config_path . 
                        '</strong>').'</p><pre>'.$this->wp_config_addon().'</pre>';
                    exit();
                    throw new FilesystemModifyException(
                        $ex->getMessage(), $ex->credentials_form(),
                        sprintf(__('Edit file %s and remove line:', 'xoogu-dynamic-shortcodes-w3tc'), '<strong>' . $config_path . 
                        '</strong>'), $config_path, 
                        $this->wp_config_addon());
                }
            }
        }
    }
    
    
    /**
     * @return string Line to add to wp-config.php to make dynamic code work in W3TC
     **/
    private function wp_config_addon() {
        return "define('W3TC_DYNAMIC_SECURITY', '".md5(mt_rand())."'); // ".__('Added by Xoogu Dynamic Shortcodes for W3 Total Cache', 'xoogu-dynamic-shortcodes-w3tc')."\r\n";
    }
    

    /**
     * Register the extension with W3TC
     * @param array $aExtensions Array of extensions installed
     * @param W3_Config $oConfig
     * @return array Array of extensions installed
     */
    function extension($aExtensions, $oConfig)
    {
        $aExtensions['xoogu-dynamic-shortcodes-w3tc'] = array (
            'name' => __('Dynamic Shortcodes', 'xoogu-dynamic-shortcodes-w3tc'),
            'author' => 'xoogu.com',
            'description' => __('Allows selected shortcodes to be generated dynamically on every page load.', 'xoogu-dynamic-shortcodes-w3tc'),
            'author uri' => 'http://www.xoogu.com/',
            'extension uri' => 'http://www.xoogu.com/dynamic-shortcodes-extension-for-w3-total-cache/',
            'extension id' => 'xoogu-dynamic-shortcodes-w3tc',
            'version' => '2015-08-11',
            'enabled' => $oConfig->get_boolean('pgcache.late_init'),
            'requirements' => sprintf(__('Late init must be enabled (see %s)', 'xoogu-dynamic-shortcodes-w3tc'), sprintf('<a href="'.strtok($_SERVER["REQUEST_URI"],'?').'?page=w3tc_pgcache#advanced">%s</a>', __('W3TC Page Cache settings', 'xoogu-dynamic-shortcodes-w3tc'))),
            //in theory should be able to define W3TC_EXTENSION_DIR to change extensions dir, defaults to wp-content/plugins
            //however, W3TC globs W3TC_DIR . '/extensions/' to find the extensions, so impossible to locate extensions in different dir to default
            'path' => 'w3-total-cache/extensions/XooguDynamicShortcodes/XooguDynamicShortcodes.php'
        );

        return $aExtensions;
    }
}

$ext = new W3_XooguDynamicShortcodesAdmin();
$ext->run();
